﻿using System;
using System.IO;
using System.Linq;
using System.Text;

namespace Wave_Reader
{
    class Program
    {
        static void Main(string[] args)
        {
            // Mehrdimensionales Array für die Chunks
            // 1. Dimension = RIFF Chunk
            // 2. Dimension = ftm Chunk
            // 3. Dimension = data Chunk
            byte[][] Chunk = new byte[3][];
            Int16[] AudioData;

            FileStream Datei;
            ASCIIEncoding Encoder = new ASCIIEncoding();

            Console.Title = "Wave Reader";
            Console.WriteLine("Wave-Dekoder Version: {0}", System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString());
            Console.WriteLine("www.kampis-elektroecke.de");
            Console.WriteLine();

            Chunk[0] = new byte[12];
            Chunk[1] = new byte[24];

            // Chunkdata
            byte[] ChunkID = new byte[4];
            Int32 ChunkSize = 0;
            byte[] Format = new byte[4];
            byte[] SubChunkID = new byte[4];
            byte[] SubChunkSize = new byte[4];
            byte[] AudioFormat = new byte[2];
            byte[] Channel = new byte[2];
            byte[] Samplerate = new byte[4];
            byte[] Byterate = new byte[4];
            byte[] BlockAlign = new byte[2];
            byte[] BitsPerSample = new byte[2];
            byte[] SubChunk2ID = new byte[4];
            byte[] SubChunk2Size = new byte[4];

            using (Datei = new FileStream(@"C:\Users\Kampi\Desktop\Wave\Test.wav", FileMode.Open))
            {
                // Read first chunk
                Datei.Read(Chunk[0], 0, 12);

                // Read second chunk
                Datei.Read(Chunk[1], 0, 24);

                // Read third chunk (hold the audiodata)
                ChunkSize = BitConverter.ToInt32(Chunk[0].Skip(4).Take(4).ToArray(), 0) + 8;
                Chunk[2] = new byte[ChunkSize];
                AudioData = new Int16[ChunkSize / 2];
                Datei.Read(Chunk[2], 0, ChunkSize);
            }

            ChunkID = Chunk[0].Take(4).ToArray();
            Format = Chunk[0].Skip(8).Take(4).ToArray();
            SubChunkID = Chunk[1].Skip(0).Take(4).ToArray();
            SubChunkSize = Chunk[1].Skip(4).Take(4).ToArray();
            AudioFormat = Chunk[1].Skip(8).Take(2).ToArray();
            Channel = Chunk[1].Skip(10).Take(2).ToArray();
            Samplerate = Chunk[1].Skip(12).Take(4).ToArray();
            Byterate = Chunk[1].Skip(16).Take(4).ToArray();
            BlockAlign = Chunk[1].Skip(20).Take(2).ToArray();
            BitsPerSample = Chunk[1].Skip(22).Take(2).ToArray();
            SubChunk2ID = Chunk[2].Take(4).ToArray();
            SubChunk2Size = Chunk[2].Skip(4).Take(4).ToArray();

            // Write .coe-File and fill it with the 16-Bit Audiodata 
            StreamWriter CoeWriter = new StreamWriter(@"C:\Users\Kampi\Desktop\Wave\AudioDaten.coe");

            CoeWriter.WriteLine(@"; Sample memory initialization file for Single Port Block Memory,");
            CoeWriter.WriteLine(@"; v3.0 or later.");
            CoeWriter.WriteLine(@";");
            CoeWriter.WriteLine(@"; This.COE file specifies initialization values for a block");
            CoeWriter.WriteLine(@"; memory of depth = 16, and width = 8.In this case, values are");  
            CoeWriter.WriteLine(@"; specified in hexadecimal format. ");
            CoeWriter.WriteLine(@"memory_initialization_radix = 16; ");  
            CoeWriter.WriteLine(@"memory_initialization_vector = ");

            string[] Data = BitConverter.ToString(Chunk[2], 0).Split('-');

            for (int i = 0; i < 250000; i = i + 2)
            {
                CoeWriter.WriteLine(Data[i+1] + Data[i] + ",");
            }

            CoeWriter.Close();

            // Output to console
            Console.WriteLine("Chunk 1");
            Console.WriteLine("Chunk ID: " + Encoder.GetString(ChunkID));
            Console.WriteLine("Größe der Wave Datei: " + Convert.ToString(ChunkSize) + " Bytes");
            Console.WriteLine("Format: " + Encoder.GetString(Format));
            Console.WriteLine("-----------------------------------------");
            Console.WriteLine("Chunk 2");
            Console.WriteLine("Chunk ID: " + Encoder.GetString(SubChunkID));
            Console.WriteLine("Chunk Size: " + BitConverter.ToInt16(SubChunkSize, 0));
            Console.WriteLine("Audio Format: " + BitConverter.ToInt16(AudioFormat, 0));
            Console.WriteLine("Anzahl Kanäle: " + BitConverter.ToInt16(Channel, 0));
            Console.WriteLine("Samplerate: " + BitConverter.ToInt32(Samplerate, 0) + " Hz");
            Console.WriteLine("Byterate: " + BitConverter.ToInt32(Byterate, 0));
            Console.WriteLine("Bytes pro Sample: " + BitConverter.ToInt16(BlockAlign, 0));
            Console.WriteLine("Bits pro Sample: " + BitConverter.ToInt16(BitsPerSample, 0));
            Console.WriteLine("-----------------------------------------");
            Console.WriteLine("Chunk 3");
            Console.WriteLine("Chunk ID: " + Encoder.GetString(SubChunk2ID));
            Console.WriteLine("Chunk Size: " + BitConverter.ToInt32(SubChunk2Size, 0));
            Console.WriteLine("Fertig...");
            Console.ReadLine();
        }
    }
}
